@@ -20,6 +20,7 @@ import time
 from urllib import parse as urlparse
 
 import requests
+from urllib3.exceptions import InsecureRequestWarning
 
 from sushy import exceptions
 from sushy.taskmonitor import TaskMonitor
@@ -37,7 +38,6 @@ class Connector(object):
         self._url = url
         self._verify = verify
         self._session = requests.Session()
-        self._session.verify = self._verify
         self._response_callback = response_callback
         self._auth = None
         self._server_side_retries = server_side_retries
@@ -68,6 +68,13 @@ class Connector(object):
                         'be removed in the future')
             self.set_http_basic_auth(username, password)
 
+        if not self._verify:
+            # As the user specifically needs to opt out of certificate
+            # validation the user is aware of the security implications
+            # and does not need to be overwhelmed by the urllib3 warnings
+            requests.packages.urllib3.disable_warnings(
+                category=InsecureRequestWarning)
+
     def set_auth(self, auth):
         """Sets the authentication mechanism for our connector."""
         self._auth = auth
@@ -87,15 +94,21 @@ class Connector(object):
 
     def check_retry_on_exception(self, exception_msg):
         """Checks whether retry on exception is required."""
-        if ('SYS518' in str(exception_msg)):
+        retry = False
+        exc_str = str(exception_msg)
+        if 'SYS518' in exc_str:
             LOG.debug('iDRAC is not yet ready after previous operation. '
-                      'Error: %(err)s', {'err': str(exception_msg)})
-            return True
-        else:
-            return False
+                      'Error: %(err)s', {'err': exc_str})
+            retry = True
+        elif 'iLO.2.15.InvalidOperationForSystemState' in exc_str:
+            LOG.debug('iLO is not ready after previous operation. '
+                      'Error: %(error)s', {'err': exc_str})
+            retry = True
+        return retry
 
     def _op(self, method, path='', data=None, headers=None, blocking=False,
-            timeout=60, **extra_session_req_kwargs):
+            timeout=60, server_side_retries_left=None,
+            **extra_session_req_kwargs):
         """Generic RESTful request handler.
 
         :param method: The HTTP method to be used, e.g: GET, POST,
@@ -105,6 +118,8 @@ class Connector(object):
         :param headers: Optional dictionary of headers.
         :param blocking: Whether to block for asynchronous operations.
         :param timeout: Max time in seconds to wait for blocking async call.
+        :param server_side_retries_left: Remaining retries. If not provided
+            will use limit provided by instance's server_side_retries
         :param extra_session_req_kwargs: Optional keyword argument to pass
          requests library arguments which would pass on to requests session
          object.
@@ -112,6 +127,10 @@ class Connector(object):
         :raises: ConnectionError
         :raises: HTTPError
         """
+
+        if server_side_retries_left is None:
+            server_side_retries_left = self._server_side_retries
+
         url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
             self._url, path)
         headers = headers or {}
@@ -133,6 +152,7 @@ class Connector(object):
         try:
             response = self._session.request(method, url, json=data,
                                              headers=headers,
+                                             verify=self._verify,
                                              **extra_session_req_kwargs)
         except requests.exceptions.RequestException as e:
             # Capture any general exception by looking for the parent
@@ -190,6 +210,7 @@ class Connector(object):
                     response = self._session.request(
                         method, url, json=data,
                         headers=headers,
+                        verify=self._verify,
                         **extra_session_req_kwargs)
                 except exceptions.HTTPError as retry_exc:
                     LOG.error("Failure occured while attempting to retry "
@@ -208,18 +229,40 @@ class Connector(object):
         except exceptions.ServerSideError as e:
             if ((method.lower() == 'get'
                 or self.check_retry_on_exception(e.message))
-                    and self._server_side_retries > 0):
+                    and server_side_retries_left > 0):
                 LOG.warning('Got server side error %s in response to a '
-                            'GET request, retrying after %d seconds',
-                            e, self._server_side_retries)
+                            'request, retrying after %d seconds. Retries '
+                            'left %d.',
+                            e, self._server_side_retries_delay,
+                            server_side_retries_left)
                 time.sleep(self._server_side_retries_delay)
-                self._server_side_retries -= 1
-                return self._op(method, path, data=data, headers=headers,
-                                blocking=blocking, timeout=timeout,
-                                **extra_session_req_kwargs)
+                server_side_retries_left -= 1
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    **extra_session_req_kwargs)
+            else:
+                raise
+        except exceptions.BadRequestError as e:
+            if (method.lower() != 'get'
+                    and self.check_retry_on_exception(e.message)
+                    and server_side_retries_left > 0):
+                LOG.warning('Server has indicated a BadRequest for %s but '
+                            'the response payload is a known retriable '
+                            'condition and we will retry in %d seconds. '
+                            'Retries left  %d.',
+                            e, self._server_side_retries_delay,
+                            server_side_retries_left)
+                time.sleep(self._server_side_retries_delay)
+                server_side_retries_left -= 1
+                return self._op(
+                    method, path, data=data, headers=headers,
+                    blocking=blocking, timeout=timeout,
+                    server_side_retries_left=server_side_retries_left,
+                    **extra_session_req_kwargs)
             else:
                 raise
-
         if blocking and response.status_code == 202:
             if not response.headers.get('Location'):
                 m = ('HTTP response for %(method)s request to %(url)s '
